#!/usr/bin/python

#   Copyright (c) 2017, Pan Labs (panorbit.in).  
#   This file is licensed under the MIT License.
#   See the LICENSE file.

import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import random
import json


#this function prints data to txt file
#it takes data and ProjectName(name of file) as input
#gets fileName.txt in output folder
def PrintIt(data,ProjectName):

    if ProjectName:
        fileName='.'.join([ProjectName,'txt'])
        path='/'.join(['output',fileName])
    else:
        fileName='.'.join(['output','txt'])
        path='/'.join(['output',fileName])

    f=open(fileName,'a')
    f.write(data)
    f.close()



#this functions prints data into json file (json format)
class Json():
    def __init__(self):
        self.PresentURL=''
        self.data=''
        self.ProjectName=''
        self.Data={}

    def jsonData(self):

        if self.PresentURL in self.Data:
            self.Data[self.PresentURL]+=' '+self.data
        else:
            self.Data[self.PresentURL]=self.data
        json_encoded=json.dumps(self.Data)

        if self.ProjectName:
            fileName='.'.join([self.ProjectName,'json'])
            path='/'.join(['output',fileName])
        else:
            fileName='.'.join(['output','json'])
            path='/'.join(['output',fileName])

        f=open(fileName,'w')
        f.write(json_encoded)
        f.close()



#   Copyright (c) 2017, Pan Labs (panorbit.in).  
#   This file is licensed under the MIT License.
#   See the LICENSE file.

import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))


from text_spider import parse



#!/usr/bin/python

#   Copyright (c) 2017, Pan Labs (panorbit.in).  
#   This file is licensed under the MIT License.
#   See the LICENSE file.


import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), '.'))

from HTMLParser import HTMLParser

from text_spider.PrintIt import PrintIt,Json


class MyParser(HTMLParser):

    def __init__(self):
        HTMLParser.__init__(self)
        #this varibales are switches to check how output_data flows
        self.inLink = False
        self.json_output=False
        self.txt_output=False

        self.dataArray = []
        self.lasttag = None
        self.lastname = None
        self.lastvalue = None

        # self.checktags=['a','p','br\\','h1','h2','h3','h4','h5','h6','title']

        #tags lists to check html content
        self.headTags=['h1','h2','h3','h4','h5','h6']
        self.textTags=['p','pre','br\\','li','ul','div','span', 'title']


        self.PagesToVisit=[]
        self.ProjectName=''
        self.PresentURL=''
        self.output_data={}
        self.get_first_page=True


        #Json object is created and an instance is assigned to self.Jsonobj
        self.Jsonobj=Json()


    #handels start tag
    def handle_starttag(self, tag, attrs):
        self.inLink = False
        # print(tag)
        if (tag.lower() in self.headTags or tag.lower() in self.textTags):
        	self.inLink = True
        	self.lasttag = tag
        # elif tag.lower()=='img':
        #     for (key, value) in attrs:
        #         if key == 'src':
        #             print ('images: %s'%(value))

        elif tag.lower()=='iframe':
            for (key, value) in attrs:
                if key == 'src':
                    # print ('videos: %s'%(value))
                    if self.txt_output==True:
                        PrintIt('videos: %s'%(value),self.ProjectName)

        elif tag.lower() == 'a':
            self.inLink = True
            self.lasttag = tag
            if self.get_first_page==True:

                for (key, value) in attrs:
                    if key == 'href':
                        if value not in self.PagesToVisit:

                            self.PagesToVisit+=[value]



    def handle_endtag(self, tag):
        if (tag.lower() in self.headTags or tag.lower() in self.textTags):
            self.inlink = False

    #this function prints data when
    def handle_data(self, data):

        if (self.lasttag in self.headTags or self.lasttag in self.textTags) and self.inLink and data.strip():

            #prints text output_data
            if self.txt_output==True:
                if self.lasttag == 'title' :
                    PrintIt("Title : ", self.ProjectName)
                    PrintIt(data,self.ProjectName)
                PrintIt(data,self.ProjectName)
                PrintIt('\n',self.ProjectName)

            #prints json output_data
            if self.json_output==True:
                self.Jsonobj.PresentURL=self.PresentURL
                self.Jsonobj.data=data
                self.Jsonobj.ProjectName=self.ProjectName
                self.Jsonobj.jsonData()


            #gets dict of url and data key/value pairs and saves in output_data
            if self.PresentURL in self.output_data:
                self.output_data[self.PresentURL]+=' ' + data + '\n'
            else:
                self.output_data[self.PresentURL] = data + '\n'


    def remove_tags(self,tags):
        for tag in tags:
            if tag in self.headTags:
                self.headTags.remove(tag)
            elif tag in self.textTags:
                self.headTags.remove(tag)

    def add_tags(self,tags):
        for tag in tags:
            if tag not in self.textTags:
                self.textTags+=[tag]



#!/usr/bin/python

#   Copyright (c) 2017, Pan Labs (panorbit.in).  
#   This file is licensed under the MIT License.
#   See the LICENSE file.

import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import requests
from text_spider.parse import MyParser
from text_spider.PrintIt import PrintIt
import urlparse



class Spider():

    #init constructor function
    #takes input URL and allowed_domain as input
    def __init__(self,url,allowed_domain):
        self.url=url
        self.allowed_domain=allowed_domain

        self.ProjectName=''#ProjectName variable which is used to get o/p file name
        # self.add_tags=[]

        #Myparser object is created and an instance is assigned to self.parser
        self.parser=MyParser()
        #the output_data which is recursed from My parser function is assigned to output_data varibale
        self.output_data_s=self.parser.output_data

        #this varibales are switches to check how output_data flows
        self.json_output=False
        self.txt_output=False
        self.is_recursive=True


    def spider(self):
        # parser = MyParser()
        links=self.parser.PagesToVisit

        #datas which is stored in spider varibale is assigned to Myparser objects
        self.parser.ProjectName=self.ProjectName
        self.parser.PresentURL=''
        self.parser.json_output=self.json_output
        self.parser.txt_output=self.txt_output
        self.parser.get_first_page=self.is_recursive
        
        if not self.url.startswith('http'):
            self.url=''.join(['http://',self.url])
        #add requested url to links
        links+=[self.url]

        visitedLinks=[]#this varibale checks links visited

        print('spider is crawling for you')

        #recursion links to parse html content starts here
        for link in links:
            try:
                #checks if url link is in domain range
                if link.startswith(self.allowed_domain):
                    #checks if url is not visited before
                    if link not in visitedLinks:
                        #gets html content using request module
                        r=requests.get(link)
                        self.parser.PresentURL=str(link)

                        if self.txt_output==True:
                            PrintIt('-----------------------------------------------------------------------------\n',self.ProjectName)
                            PrintIt('URL : %s \n'%(link),self.ProjectName)
                        self.parser.feed(r.content)
                        visitedLinks+=[link]
                        links=self.parser.PagesToVisit
                elif not link.startswith('http'):
                    if link not in visitedLinks:
                        if not link.startswith('#'):
                            joinedLink=urlparse.urljoin(self.allowed_domain,link)
                            r=requests.get(joinedLink)
                            self.parser.PresentURL=str(link)
                            # print('from spider2')
                            if self.txt_output==True:
                                PrintIt('-----------------------------------------------------------------------------\n',self.ProjectName)
                                PrintIt('URL : %s \n'%(joinedLink),self.ProjectName)
                            self.parser.feed(r.content)
                            visitedLinks+=[link]
                            links=self.parser.PagesToVisit
            except:
                pass
                #print('.')
        print('crawling completed successfully')
        return self.output_data_s,links #returns output_data and links


#this function add tags to lists
#takes tags as input in list []
    def add_tags(self,tags):
        for tag in tags:
            if tag not in self.parser.textTags:
                self.parser.textTags+=[tag]


#this function remove tags from lists
#takes tags as input in list []
    def remove_tags(self,tags):
        for tag in tags:
            if tag in self.parser.headTags:
                self.parser.headTags.remove(tag)
            elif tag in self.parser.textTags:
                self.parser.textTags.remove(tag)

#this function gets all takes present in lists
    def get_tags_list(self):
        if self.parser.checktags:
            return(self.parser.textTags+self.parser.headTags)



#   Copyright (c) 2017, Pan Labs (panorbit.in).  
#   This file is licensed under the MIT License.
#   See the LICENSE file.

from setuptools import setup,find_packages

setup(
	name='text_spider',
	version='1.0.1',
	description='text scraping spider',
	url='http://panorbit.in/',
	author="prajwal",
	author_email="labs@panorbit.in",
	license="MIT",
	packages=find_packages(),
        classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
         ],
         install_requires=['requests'],
         scripts=['scripts/text-spider'],

	)
